/*Copyright (C) 2017 Abashin Valerii Gennadievich <valeriy@abashin.ru>

This file is part of abashin.ru software.

Abashin.ru is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

Abashin.ru is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with Abashin.ru. If not, see <http://www.gnu.org/licenses/>.
*/

#include <iostream>
#include <fstream>
#include <cstdlib>
#include <vector>
#include <ctime>

using namespace std;

struct HUMAN_STATUS {
  time_t current_time;
  int intensity;
  float pfs;
  int device_id;
};

struct DOUBLE_ETALON {
  time_t current_time;
  int intensity;
  int key1;
  int retaining1;
  int space1;
  int key2;
  int retaining2;
};

struct MOUSEPRESSETALON {
  time_t current_time;
  int intensity;
  int key;
  int retaining1;
};

struct KEYBOARD_DOUBLE_ETALON {
  int key1;
  int retaining1_min;
  int retaining1_max;
  int space1_min;
  int space1_max;
  int key2;
  int retaining2_min;
  int retaining2_max;
};

struct MOUSE_PRESS_ETALON {
  int key;
  int retaining1_min;
  int retaining1_max;
};

int calculate_human_status (struct HUMAN_STATUS * save_hs,
     struct HUMAN_STATUS * current_hs, vector <MOUSE_PRESS_ETALON> * me,
     vector <KEYBOARD_DOUBLE_ETALON> * ke, vector<MOUSEPRESSETALON> * mouse_data,
     vector<DOUBLE_ETALON> * keyboard_data);


int main (int argc, char * argv[]) {
  string buff;
  if (argc != 5) return 0;
  char * SAVEDATA = argv[1];
  char * ETALON = argv[2];
  char * HUMANSTATUS = argv[3];
  char * HUMANSTATUSLOG = argv[4];

  vector <MOUSE_PRESS_ETALON> me;
  me.reserve(10);
  struct MOUSE_PRESS_ETALON mpe = {0};
  vector <KEYBOARD_DOUBLE_ETALON> ke;
  ke.reserve(1000);
  struct KEYBOARD_DOUBLE_ETALON kde = {0};

  ifstream etalon_file;
  buff.clear();
  buff = buff + ETALON;
  etalon_file.open(buff, ifstream::in);
  if (!etalon_file) return 0;
  time_t current_time = 0;
  etalon_file >> current_time;
  int temp = 0;
  while (etalon_file.good()) {
    etalon_file >> temp;
    if (temp == 2) {
      etalon_file >> mpe.key >> mpe.retaining1_min >> mpe.retaining1_max;
      me.push_back(mpe);
    }
    if (temp == 1) {
      etalon_file >> kde.key1 >>
      kde.retaining1_min >> kde.retaining1_max >> kde.space1_min >>
      kde.space1_max >> kde.key2 >> kde.retaining2_min >> kde.retaining2_max;
      ke.push_back(kde);
    }
  }
  etalon_file.close();

  vector<MOUSEPRESSETALON> mouse_data;
  mouse_data.reserve(1000);
  vector<DOUBLE_ETALON> keyboard_data;
  keyboard_data.reserve(1000);

  ifstream save_data_file;
  buff.clear();
  buff = buff + SAVEDATA;
  save_data_file.open(buff, ifstream::in);

  save_data_file.seekg (0, save_data_file.end);
  int file_length = save_data_file.tellg();
  save_data_file.seekg (0, save_data_file.beg);

  char buffer[256] = {0};
  if (file_length > 4000) {
    save_data_file.seekg (-4000, save_data_file.end);
    save_data_file.getline(buffer, 256);
  }

  time_t limit_time = time(NULL) - 360;
  int code = 0;
  int previous_code = 0;
  int intensity = 0;
  while (save_data_file.good()) {
    save_data_file.getline(buffer, 256);
    if (buffer[0] != '0') continue;
    current_time = atoi((char *)&buffer[2]);
    if (current_time > limit_time) break;
  }

  while (save_data_file.good()) {
    save_data_file >> code;
    if (code == 0) save_data_file >> current_time;
    if (code == 1)
      if (code != previous_code) {
        save_data_file >> intensity;
      }
      else {
        struct DOUBLE_ETALON sde = {0};
        save_data_file >> sde.intensity;
        if (!save_data_file) {
          previous_code = code;
          continue;
        }
        save_data_file >> sde.key1;
        if (!save_data_file) {
          previous_code = code;
          continue;
        }
        save_data_file >> sde.retaining1;
        if (!save_data_file) {
          previous_code = code;
          continue;
        }
        save_data_file >> sde.space1;
        if (!save_data_file) {
          previous_code = code;
          continue;
        }
        save_data_file >> sde.key2;
        if (!save_data_file) {
          previous_code = code;
          continue;
        }
        save_data_file >> sde.retaining2;
        if (!save_data_file) {
          previous_code = code;
          continue;
        }
        sde.current_time = current_time;
        if (!save_data_file) {
          previous_code = code;
          continue;
        }
        keyboard_data.push_back(sde);
      }
    else;
    if (code == 2)
      if (code != previous_code) {
        save_data_file >> intensity;
      }
      else {
        struct MOUSEPRESSETALON sme = {0};
        save_data_file >> sme.intensity;
        if (!save_data_file) {
          previous_code = code;
          continue;
        }
        save_data_file >> sme.key;
        if (!save_data_file) {
          previous_code = code;
          continue;
        }
        save_data_file >> sme.retaining1;
        if (!save_data_file) {
          previous_code = code;
          continue;
        }
        sme.current_time = current_time;
        if (!save_data_file) {
          previous_code = code;
          continue;
        }
        mouse_data.push_back(sme);
      }
    else;
    previous_code = code;
  }
  save_data_file.close();

  struct HUMAN_STATUS save_hs = {0};
  struct HUMAN_STATUS hs = {0};

  fstream human_status_file;
  buff.clear();
  buff = buff + HUMANSTATUS;
  human_status_file.open(buff, fstream::in | fstream::out | fstream::app);
  if (human_status_file) {
    human_status_file.seekg (0, human_status_file.beg);
    human_status_file >> save_hs.current_time;
    human_status_file >> save_hs.intensity;
    human_status_file >> save_hs.pfs;
    human_status_file >> save_hs.device_id;
  }
  human_status_file.close();

  temp = calculate_human_status ((struct HUMAN_STATUS *) &save_hs,
     (struct HUMAN_STATUS *) &hs, (vector <MOUSE_PRESS_ETALON> *) &me,
     (vector <KEYBOARD_DOUBLE_ETALON> *) &ke, (vector<MOUSEPRESSETALON> *) &mouse_data,
     (vector<DOUBLE_ETALON> *) &keyboard_data);

  buff.clear();
  buff = buff + HUMANSTATUS;
  human_status_file.open(buff, fstream::out | fstream::trunc);
  human_status_file << hs.current_time << " " << hs.intensity <<
                       " " << hs.pfs << " " << hs.device_id;
  human_status_file.close();

  buff.clear();
  buff = buff + HUMANSTATUSLOG;
  human_status_file.open(buff, fstream::out | fstream::app);
  human_status_file << hs.current_time << " " << hs.intensity <<
                       " " << hs.pfs << " " << hs.device_id << endl;
  human_status_file.close();

  return 0;
}


int calculate_human_status (struct HUMAN_STATUS * save_hs,
     struct HUMAN_STATUS * current_hs, vector <MOUSE_PRESS_ETALON> * me,
     vector <KEYBOARD_DOUBLE_ETALON> * ke, vector<MOUSEPRESSETALON> * mouse_data,
     vector<DOUBLE_ETALON> * keyboard_data) {
  current_hs->current_time = time(NULL);
  current_hs->device_id = 0;

  current_hs->intensity = 0;
  for (vector<MOUSEPRESSETALON>::iterator count = mouse_data->begin();
    count!=mouse_data->end(); count++) {
    if (count->current_time > current_hs->current_time - 60) {
      current_hs->intensity += count->intensity;
    }
  }
  for (vector<DOUBLE_ETALON>::iterator count = keyboard_data->begin();
    count!=keyboard_data->end(); count++) {
    if (count->current_time > current_hs->current_time - 60) {
      current_hs->intensity += count->intensity;
    }
  }

  current_hs->pfs = 0.0;
  float pfs = 0.0;
  int value_count = 0;
  for (vector<MOUSEPRESSETALON>::iterator count = mouse_data->begin();
    count!=mouse_data->end(); count++) {
    for (vector<MOUSE_PRESS_ETALON>::iterator me_count = me->begin();
      me_count!=me->end(); me_count++) {
      if (me_count->key == count->key) {
        if(count->retaining1 > me_count->retaining1_max) {
          pfs += 0.0;
          value_count++;
          break;
        }
        if(count->retaining1 < me_count->retaining1_min) {
          pfs += 1.0;
          value_count++;
          break;
        }
        int temporary_value = me_count->retaining1_max -
                              me_count->retaining1_min;
        if (temporary_value == 0) temporary_value = 1;
        pfs += 1.0 - (float)(count->retaining1 -
                        me_count->retaining1_min)/(float)temporary_value;
        value_count++;
        break;
      }
    }
  }

  for (vector<DOUBLE_ETALON>::iterator count = keyboard_data->begin();
    count!=keyboard_data->end(); count++) {
    for (vector<KEYBOARD_DOUBLE_ETALON>::iterator ke_count = ke->begin();
      ke_count!=ke->end(); ke_count++) {
      if ((ke_count->key1 == count->key1) && (ke_count->key2 == count->key2)) {
        int temporary_value = 0;
        if (count->retaining1 > ke_count->retaining1_max) {
          pfs += 0.0;
          value_count++;
        }
        if (count->retaining1 < ke_count->retaining1_min) {
          pfs += 1.0;
          value_count++;
        }
        if ((count->retaining1 >= ke_count->retaining1_min) &&
           (count->retaining1 <= ke_count->retaining1_max)) {
          temporary_value = ke_count->retaining1_max -
                            ke_count->retaining1_min;
          if (temporary_value == 0) temporary_value = 1;
          pfs += 1.0 - (float)(count->retaining1 -
                ke_count->retaining1_min)/(float)temporary_value;
          value_count++;
        }
        if (count->space1 > ke_count->space1_max) {
          pfs += 0.0;
          value_count++;
        }
        if (count->space1 < ke_count->space1_min) {
          pfs += 1.0;
          value_count++;
        }
        if ((count->space1 <= ke_count->space1_max) &&
            (count->space1 >= ke_count->space1_min)) {
          temporary_value = ke_count->space1_max - ke_count->space1_min;
          if (temporary_value == 0) temporary_value = 1;
          pfs += 1.0 - (float)(count->space1 -
                ke_count->space1_min)/(float)temporary_value;
          value_count++;
        }
        if (count->retaining2 > ke_count->retaining2_max) {
          pfs += 0.0;
          value_count++;
        }
        if (count->retaining2 < ke_count->retaining2_min) {
          pfs += 1.0;
          value_count++;
        }
        if ((count->retaining2 <= ke_count->retaining2_max) &&
            (count->retaining2 >= ke_count->retaining2_min)) {
          temporary_value = ke_count->retaining2_max -
                            ke_count->retaining2_min;
          if (temporary_value == 0) temporary_value = 1;
          pfs += 1.0 - (float)(count->retaining2 -
                ke_count->retaining2_min)/(float)temporary_value;
          value_count++;
        }
      }
    }
  }
  if (value_count == 0) value_count = 1;
  pfs = pfs / (float)value_count;
  current_hs->pfs = pfs;

  return 0;
};
