#ifndef ABASHIN_RU_TASKS_H
#define ABASHIN_RU_TASKS_H

#include <ctime>
#include <fstream>
#include <algorithm>
#include <string>
#include <cstring>
#include <list>

#define CREATE_TIME "[create time]"
#define ID "[id]"
#define LABEL "[label]"
#define START_TIME "[start time]"
#define STOP_TIME "[stop time]"
#define PRIORITY "[priority]"
#define SIGNAL "[signal]"
#define STATUS "[status]"
#define TEXT "[text]"
#define SKILL "[skill]"

#define TEXT_NONE "none"

#define NONE 0
#define HIGH 1
#define MIDDLE 2
#define LOW 3

#define TASK_STATUS_ON "on"
#define TASK_STATUS_OFF "off"

using namespace std;

struct task {
  time_t create_time;
  string id;
  string label;
  time_t start_time;
  time_t stop_time;
  int priority;
  string signal;
  string status;
  string text;
  string skill;
};

struct task_record {
  time_t create_time;
  string id;
  string status; // on/off
};

class tasks {
public:
  tasks ();
  tasks (const string * tasks_filename);
  tasks (const string * tasks_filename, const string * task_record_filename);
  ~tasks ();
  void set_file(const string * tasks_filename, const string * task_record_filename);
  void load_tasks(void);
  void load_tasks(const string * tasks_filename);
  void save_tasks(void);
  void save_tasks(const string * tasks_filename);
  void load_tasks_record(void);
  void load_tasks_record(const string * tasks_record_filename);
  void save_tasks_record(void);
  void save_tasks_record(const string * tasks_record_filename);
  void create_task(struct task * task_add);
  void create_skill(const string * skill);
  void create_list_of_skill();
  void change_the_name_of_the_skill(const string * old_name, const string * new_name);
  void merge_skill(const string * basic_name, const string * join_name);
  void delete_skill(string * skill_name);
  void delete_task(const struct task * task_del);
  void delete_local_tasks();
  void delete_tasks_skill(string * task_name);
  void delete_all_tasks();
  struct task * show_tasks(struct task * st);
  void show_tasks_reset();
  string * show_skill(string * show_skill);
  void show_skill_reset();
  char change_the_task (int number, struct task * chtask);
  void activate_task (string * task_id);
  void deactivate_task (void);
  string * show_active_task (string * active_task_id);

private:
  void control_id ();
  string filename;
  string file_task_record;
  list <task> list_tasks;
  list <string> list_skills;
  list <task_record> list_task_records;
  
  list <string>::iterator show_skills_iterator;
  list <task>::iterator show_list_tasks_iterator;
  list <task_record>::iterator show_list_tasks_record_iterator;
};
#endif

/*
File format:
[create time]
Time
[id]
Number ID
[label]
Label
[start time]
Start time (time_t)
[stop time]
Stop time (time_t)
[priority]
high/middle/low
[signal]
Path signal mp3
[status]
status
[text]
Text
[skill]
Skill

File format:
create_time id status
*/
