#include <dirent.h>
#include <sys/types.h> 
#include <sys/stat.h>
#include <stdio.h>
#include <unistd.h> /*getpid*/
#include <stdlib.h> /*mkstemp(),*/
#include <string.h>
#include <pthread.h>

/*Copyright (C) 2017 Abashin Valerii Gennadievich <valeriy@abashin.ru>

This file is part of abashin.ru software.

Abashin.ru is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

Abashin.ru is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with Abashin.ru. If not, see <http://www.gnu.org/licenses/>.
*/

#include <signal.h>
#include <sys/wait.h>

#define true 1
#define false 0

#define MOUSESPATH "/dev/input"
#define MOUSES 5
#define MOUSEPATHSIZE 100
#define TEMPFILENAMESIZE 30
#define TEMPFILENAME "/tmp/abashin.ru_mouses"
#define ETALONFILENAME "/tmp/abashin.ru_mouses_etalon"
#define FINDLOOPDURATION 60

sig_atomic_t child_exit_status;
pthread_t ptwaiter_id;
int sig_num;

struct mouse {
  pid_t pid;
  char mouse_path[MOUSEPATHSIZE];
};

int reset_disabled_mouses(struct mouse * mouses);
int find_mouses_and_start (char * device_path, char * tempfilename,
                              struct mouse * mouses);
int file_exists (char * str_filename);
void clean_up_child_process (int signal_number);
void * waiter (void * empty);
char * out_posix_directory (char * path);
char * in_posix_directory (char * path, char * directory_name);

int main(int argc, char * argv[]) {
  struct mouse mouse_massive [MOUSES] = {0};

  char tempfilename [TEMPFILENAMESIZE] = {0};
  strcpy ((char *)&tempfilename, TEMPFILENAME);
  FILE * temp_file = fopen((char *)&tempfilename,"w");
  if (temp_file == NULL) {
    fprintf(stderr, "Can`t create temp file.\n");
    return 0;
  }
  fclose(temp_file);
  chmod((char*)&tempfilename, 0700);

  struct sigaction sigchld_action = {0};
  sigchld_action.sa_handler = &clean_up_child_process;
  sigaction (SIGCHLD, &sigchld_action, NULL);

  while(1) {
    int result = 0;
    result = reset_disabled_mouses((struct mouse *) &mouse_massive);
    if (result) fprintf(stderr, "Can`t reset mouses.\n");
    char path[100] = {0};
    strcpy((char *)&path, MOUSESPATH);
    result = find_mouses_and_start ((char *)&path, (char *)&tempfilename,
                                       (struct mouse *) &mouse_massive);
    if (result) fprintf(stderr, "Can`t find and start mouses.\n");
    sleep(FINDLOOPDURATION);
    pid_t pid = fork();
    if (pid < 0) {
      fprintf(stderr, "Fork create etalon mouses failed\n");
      exit(-1);
    }
    else
      if (pid == 0) {
        execlp("/usr/sbin/mouse_etalon","mouse_etalon",
               tempfilename, ETALONFILENAME, NULL);
      }
      else {
        int status;
        wait(&status);
      }
    chmod(ETALONFILENAME, 0744);
    temp_file = fopen((char *)&tempfilename,"w");
    if (temp_file == NULL) {
      fprintf(stderr, "Can`t create mouses temp file.\n");
      return 0;
    }
    fclose(temp_file);
  }
  return 0;
}

int reset_disabled_mouses(struct mouse * mouses) {
  int count;
  for (count = 0; count < MOUSES; count++) {
    if (mouses[count].pid != 0) {
      FILE * mouse_file = fopen((char *)&mouses[count].mouse_path,"r");
      if (mouse_file == NULL) {
        kill (mouses[count].pid, SIGKILL);
        bzero(&mouses[count].mouse_path, sizeof(char)*MOUSEPATHSIZE);
      }
      else fclose (mouse_file);
      if (kill (mouses[count].pid, 0) != 0) {
        mouses[count].pid = 0;
        bzero(&mouses[count].mouse_path, sizeof(char)*MOUSEPATHSIZE);
      }
    }
  }
  return 0;
};

int find_mouses_and_start (char * device_path, char * tempfilename,
                             struct mouse * mouses) {
  DIR *dir;
  struct dirent *ent;
  struct stat statbuf;
  dir = opendir(device_path);
  if (dir == 0) return 0;
  while ((ent=readdir(dir)) != 0) {
    if ((strcmp(ent->d_name, ".") != 0) &&
      (strcmp(ent->d_name, "..") != 0)) {
      memset((void*)&statbuf,0,sizeof(statbuf));
      char full_path[1000] = {0};
      strcpy((char *)&full_path, device_path);
      strcat((char *)&full_path, ent->d_name);
      stat((char *)&full_path, &statbuf);
      if ((strstr(ent->d_name, "event") != NULL) &&
          (strstr(ent->d_name, "mouse") != NULL)) {
        char mouse_path[MOUSEPATHSIZE] = {0};
        strcpy((char *)&mouse_path, device_path);
        strcat((char *)&mouse_path, ent->d_name);
        char find_flag = 0;
        int count = 0;
        for (count = 0; count < MOUSES; count++) {
          if (strcmp ((char *)&mouse_path,
             (char *)&mouses[count].mouse_path) == 0) {
            find_flag = 1;
            break;
          }
        }
        if (find_flag == 0) {
          for (count = 0; count < MOUSES; count++) {
            if (mouses[count].pid == 0) {
              strcpy((char *)&mouses[count].mouse_path, (char *)&mouse_path);
              pid_t pid = fork();
              if (pid < 0) {
                fprintf(stderr, "Fork new mouse failed\n");
                exit(-1);
              }
              else
                if (pid == 0) {
                  execlp("/usr/sbin/mouse_client","mouse_client",
                        (char *)&mouses[count].mouse_path,
                        tempfilename, NULL);
                }
                else mouses[count].pid = pid;
                break;
            }
          }
        }
      }
      if (ent->d_type == DT_DIR) {
        in_posix_directory(device_path, ent->d_name);
        find_mouses_and_start(device_path, tempfilename, mouses);
        out_posix_directory(device_path);
      }
    }
  }
  closedir(dir);
  return 0;
};

void clean_up_child_process (int signal_number) {
  sig_num = signal_number;
  pthread_create (&ptwaiter_id, NULL, &waiter, NULL);
};

void * waiter (void * empty) {
  int status;
  wait(&status);
  child_exit_status = status;
  return empty;
}

char * in_posix_directory (char * path, char * directory_name) {
  int count = 0;
  while (path[count] != 0) count++;
  if (path[count-1] != '/') {
    path[count] = '/';
    path[count+1] = 0;
  }
  strcat(path, directory_name);
  strcat(path, "/");
  return path;
}

char * out_posix_directory (char * path) {
  int count = 0;
  while (path[count] != 0) count++;
  count--;
  if (path[count] == '/') {
    path[count] = 0;
    count--;
  }
  while (path[count] != '/') count--;
  path[count+1] = 0;
  return path;
}

int file_exists (char * str_filename) {
  struct stat st_file_info;
  int bln_return;
  int int_stat;

  int_stat = stat(str_filename,&st_file_info);
  if(int_stat == 0) {
    bln_return = true;
  } else {
    bln_return = false;
  }

  return(bln_return);
}
